@@ -1,5 +1,5 @@
 package Gtk3;
-$Gtk3::VERSION = '0.017';
+$Gtk3::VERSION = '0.023';
 use strict;
 use warnings;
 use Carp qw/croak/;
@@ -151,8 +151,8 @@ my %_GDK_EVENT_TYPE_TO_PACKAGE = (
   'expose' => 'Expose',
   'motion-notify' => 'Motion',
   'button-press' => 'Button',
-  'button-2press' => 'Button',
-  'button-3press' => 'Button',
+  '2button-press' => 'Button',
+  '3button-press' => 'Button',
   'button-release' => 'Button',
   'key-press' => 'Key',
   'key-release' => 'Key',
@@ -186,6 +186,9 @@ my %_GDK_EVENT_TYPE_TO_PACKAGE = (
   'touch-update' => 'Touch',
   'touch-end' => 'Touch',
   'touch-cancel' => 'Touch',
+  # added in 3.6:
+  'double-button-press' => 'Button',
+  'triple-button-press' => 'Button',
 );
 
 # Make all of the above sub-types inherit from Gtk3::Gdk::Event.
@@ -1422,8 +1425,17 @@ sub Gtk3::Gdk::Pixbuf::new_from_xpm_data {
     $class, $real_data);
 }
 
-# The next three subs might have to change when
-# <https://bugzilla.gnome.org/show_bug.cgi?id=670372> is accepted.
+# Version check for the new annotations described in
+# <https://bugzilla.gnome.org/show_bug.cgi?id=670372>.
+my $HAVE_GDK_PIXBUF_2_31_2 = sub {
+  return (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () > 31) ||
+         (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () == 31 && Gtk3::Gdk::PIXBUF_MICRO () >= 2);
+};
+my $HAVE_GDK_PIXBUF_2_31_3 = sub {
+  return (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () > 31) ||
+         (Gtk3::Gdk::PIXBUF_MAJOR () == 2 && Gtk3::Gdk::PIXBUF_MINOR () == 31 && Gtk3::Gdk::PIXBUF_MICRO () >= 3);
+};
+
 
 sub Gtk3::Gdk::Pixbuf::save {
   my ($pixbuf, $filename, $type, @rest) = @_;
@@ -1432,8 +1444,18 @@ sub Gtk3::Gdk::Pixbuf::save {
     croak ("Usage: \$pixbuf->save (\$filename, \$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save (\$filename, \$type, \$key1 => \$value1, ...)");
   }
+  my $method;
+  if ($HAVE_GDK_PIXBUF_2_31_3->()) {
+      $method = 'savev';
+  }
+  elsif ($HAVE_GDK_PIXBUF_2_31_2->()) {
+      $method = 'save';
+  }
+  else {
+      $method = 'savev';
+  }
   Glib::Object::Introspection->invoke (
-    $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'savev',
+    $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
     $pixbuf, $filename, $type, $keys, $values);
 }
 
@@ -1444,9 +1466,19 @@ sub Gtk3::Gdk::Pixbuf::save_to_buffer {
     croak ("Usage: \$pixbuf->save_to_buffer (\$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save_to_buffer (\$type, \$key1 => \$value1, ...)");
   }
+  my $method;
+  if ($HAVE_GDK_PIXBUF_2_31_3->()) {
+      $method = 'save_to_bufferv';
+  }
+  elsif ($HAVE_GDK_PIXBUF_2_31_2->()) {
+      $method = 'save_to_buffer';
+  }
+  else {
+      $method = 'save_to_bufferv';
+  }
   my (undef, $buffer) =
     Glib::Object::Introspection->invoke (
-      $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'save_to_bufferv',
+      $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
       $pixbuf, $type, $keys, $values);
   return $buffer;
 }
@@ -1458,8 +1490,18 @@ sub Gtk3::Gdk::Pixbuf::save_to_callback {
     croak ("Usage: \$pixbuf->save_to_callback (\$save_func, \$user_data, \$type, \\\@keys, \\\@values)\n",
            " -or-: \$pixbuf->save_to_callback (\$save_func, \$user_data, \$type, \$key1 => \$value1, ...)");
   }
+  my $method;
+  if ($HAVE_GDK_PIXBUF_2_31_3->()) {
+      $method = 'save_to_callbackv';
+  }
+  elsif ($HAVE_GDK_PIXBUF_2_31_2->()) {
+      $method = 'save_to_callback';
+  }
+  else {
+      $method = 'save_to_callbackv';
+  }
   Glib::Object::Introspection->invoke (
-    $_GDK_PIXBUF_BASENAME, 'Pixbuf', 'save_to_callbackv',
+    $_GDK_PIXBUF_BASENAME, 'Pixbuf', $method,
     $pixbuf, $save_func, $user_data, $type, $keys, $values);
 }
 
@@ -1485,10 +1527,10 @@ if ($^V ge v5.20.0) {
   no warnings 'redefine';
 
   my $disable_setlocale = 0;
-  my $orig = \&Gtk3::disable_setlocale;
-  *{Gtk3::disable_setlocale} = sub {
+  *{'Gtk3::disable_setlocale'} = sub {
     $disable_setlocale = 1;
-    $orig->(@_);
+    Glib::Object::Introspection->invoke (
+      $_GTK_BASENAME, undef, 'disable_setlocale', @_);
   };
 
   # These two already have overrides.
@@ -1502,7 +1544,6 @@ if ($^V ge v5.20.0) {
     };
   }
 
-  # These do not.
   foreach my $function (qw/init_with_args parse_args/) {
     *{'Gtk3::' . $function} = sub {
       if (!$disable_setlocale) {
@@ -1663,8 +1704,8 @@ have as many options anymore as they had in Gtk2.  Changes to arguments will
 not be propagated to the next signal handler, and only the updated position can
 and must be returned.
 
-=item * Gtk3::Menu: The position callback passed to popup() does not receive x
-and y parameters anymore.
+=item * Gtk3::Menu: In gtk+ < 3.16, the position callback passed to popup()
+does not receive x and y parameters.
 
 =item * Gtk3::RadioAction: The constructor now follows the C API.
 
